@@ -34,7 +34,7 @@
 
 #ident "$Copyright: (c) 1980, 1990 Regents of the University of California. $"
 #ident "$Copyright: All rights reserved. $"
-#ident "$Id: quotaops.c,v 1.11 2004/01/07 15:15:31 jkar8572 Exp $"
+#ident "$Id: quotaops.c,v 1.14 2004/05/24 19:39:15 jkar8572 Exp $"
 
 #include <rpc/rpc.h>
 #include <sys/types.h>
@@ -96,7 +96,7 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 #if defined(BSD_BEHAVIOUR)
 	int j, ngroups;
 	uid_t euid;
-	gid_t gidset[NGROUPS];
+	gid_t gidset[NGROUPS], *gidsetp;
 	char name[MAXNAMELEN];
 #endif
 
@@ -107,21 +107,37 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 				euid = geteuid();
 				if (euid != id && euid != 0) {
 					uid2user(id, name);
-					die(1, _("%s (uid %d): Permission denied\n"),  name, id);
+					errstr(_("%s (uid %d): Permission denied\n"), name, id);
+					return (struct dquot *)NULL;
 				}
 				break;
 			case GRPQUOTA:
-				ngroups = getgroups(NGROUPS, gidset);
+				if (geteuid() == 0)
+					break;
+				ngroups = sysconf(_SC_NGROUPS_MAX);
+				if (ngroups > NGROUPS) {
+					gidsetp = malloc(ngroups * sizeof(gid_t));
+					if (!gidsetp) {
+						errstr(_("%s: gid set allocation (%d): %s\n"), name, ngroups, strerror(errno));
+						return (struct dquot *)NULL;
+					}
+				}
+				else
+					gidsetp = &gidset[0];
+				ngroups = getgroups(ngroups, gidsetp);
 				if (ngroups < 0) {
-					die(1, _("Error while trying getgroups(): %s\n"), strerror(errno));
-					continue;
+					if (gidsetp != gidset)
+						free(gidsetp);
+					errstr(_("%s: error while trying getgroups(): %s\n"), name, strerror(errno));
+					return (struct dquot *)NULL;
 				}
 
 				for (j = 0; j < ngroups; j++)
-					if (id == gidset[j])
+					if (id == gidsetp[j])
 						break;
-
-				if (j >= ngroups && geteuid() != 0) {
+				if (gidsetp != gidset)
+					free(gidsetp);
+				if (j >= ngroups) {
 					gid2group(id, name);
 					errstr(_("%s (gid %d): Permission denied\n"),
 						name, id);
@@ -136,8 +152,8 @@ struct dquot *getprivs(qid_t id, struct quota_handle **handles, int quiet)
 		if (!(q = handles[i]->qh_ops->read_dquot(handles[i], id))) {
 			/* If rpc.rquotad is not running filesystem might be just without quotas... */
 			if (errno != ENOENT && (errno != ECONNREFUSED || !quiet))
-				errstr(_("Error while getting quota from %s for %u: %s\n"),
-					handles[i]->qh_quotadev, id, strerror(errno));
+				errstr(_("%s: error while getting quota from %s for %u: %s\n"),
+					name, handles[i]->qh_quotadev, id, strerror(errno));
 			continue;
 		}
 		if (qhead == NULL)
@@ -240,7 +256,7 @@ int writeprivs(struct dquot *qlist, int outfd, char *name, int quotatype)
 
 #if defined(ALT_FORMAT)
 	fprintf(fd, _("Disk quotas for %s %s (%cid %d):\n"),
-		type2name(quotatype), name, *type2name(quotatype), name2id(name, quotatype));
+		type2name(quotatype), name, *type2name(quotatype), name2id(name, quotatype, NULL));
 
 	fprintf(fd,
 		_("  Filesystem                   blocks       soft       hard     inodes     soft     hard\n"));
@@ -424,7 +440,7 @@ int writeindividualtimes(struct dquot *qlist, int outfd, char *name, int quotaty
 		die(1, _("Can't duplicate descriptor of file to write to: %s\n"), strerror(errno));
 
 	fprintf(fd, _("Times to enforce softlimit for %s %s (%cid %d):\n"),
-		type2name(quotatype), name, *type2name(quotatype), name2id(name, quotatype));
+		type2name(quotatype), name, *type2name(quotatype), name2id(name, quotatype, NULL));
 	fprintf(fd, _("Time units may be: days, hours, minutes, or seconds\n"));
 	fprintf(fd,
 		_("  Filesystem                         block grace               inode grace\n"));
