@@ -17,6 +17,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <ctype.h>
+#include <paths.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/vfs.h>
@@ -32,20 +33,41 @@
 #include "quotaio_v2.h"
 
 #define min(x,y) (((x) < (y)) ? (x) : (y))
-#define CORRECT_FSTYPE(type) \
-((!strcmp(type, MNTTYPE_EXT2)) || \
-(!strcmp(type, MNTTYPE_EXT3)) || \
-(!strcmp(type, MNTTYPE_MINIX)) || \
-(!strcmp(type, MNTTYPE_UFS)) || \
-(!strcmp(type, MNTTYPE_UDF)) || \
-(!strcmp(type, MNTTYPE_REISER)) || \
-(!strcmp(type, MNTTYPE_XFS)) || \
-(!strcmp(type, MNTTYPE_NFS)))
 
 static char extensions[MAXQUOTAS + 2][20] = INITQFNAMES;
 static char *basenames[] = INITQFBASENAMES;
 static char *fmtnames[] = INITQFMTNAMES;
 
+/*
+ *	Check whether give filesystem type is supported
+ */
+
+static int correct_fstype(char *type)
+{
+	char *mtype = sstrdup(type), *next;
+
+	type = mtype;
+	do {
+		next = strchr(type, ',');
+		if (next)
+			*next = 0;
+		if (!strcmp(type, MNTTYPE_EXT2) ||
+		    !strcmp(type, MNTTYPE_EXT3) ||
+		    !strcmp(type, MNTTYPE_MINIX) ||
+		    !strcmp(type, MNTTYPE_UFS) ||
+		    !strcmp(type, MNTTYPE_UDF) ||
+		    !strcmp(type, MNTTYPE_REISER) ||
+		    !strcmp(type, MNTTYPE_XFS) ||
+		    !strcmp(type, MNTTYPE_NFS)) {
+			free(mtype);
+			return 1;
+		}
+		type = next+1;	
+	} while (next);
+	free(mtype);
+	return 0;
+}
+
 /*
  *	Convert type of quota to written representation
  */
@@ -57,18 +79,26 @@ char *type2name(int type)
 /*
  *	Convert name to uid
  */
-uid_t user2uid(char *name)
+uid_t user2uid(char *name, int *err)
 {
 	struct passwd *entry;
 	uid_t ret;
 	char *errch;
 
+	if (err)
+		*err = 0;
 	ret = strtol(name, &errch, 0);
 	if (!*errch)		/* Is name number - we got directly uid? */
 		return ret;
 	if (!(entry = getpwnam(name))) {
-		errstr(_("User %s doesn't exist.\n"), name);
-		exit(1);
+		if (!err) {
+			errstr(_("User %s doesn't exist.\n"), name);
+			exit(1);
+		}
+		else {
+			*err = -1;
+			return 0;
+		}
 	}
 	return entry->pw_uid;
 }
@@ -76,18 +106,26 @@ uid_t user2uid(char *name)
 /*
  *	Convert group name to gid
  */
-gid_t group2gid(char *name)
+gid_t group2gid(char *name, int *err)
 {
 	struct group *entry;
 	gid_t ret;
 	char *errch;
 
+	if (err)
+		*err = 0;
 	ret = strtol(name, &errch, 0);
 	if (!*errch)		/* Is name number - we got directly gid? */
 		return ret;
 	if (!(entry = getgrnam(name))) {
-		errstr(_("Group %s doesn't exist.\n"), name);
-		exit(1);
+		if (!err) {
+			errstr(_("Group %s doesn't exist.\n"), name);
+			exit(1);
+		}
+		else {
+			*err = -1;
+			return 0;
+		}
 	}
 	return entry->gr_gid;
 }
@@ -95,12 +133,12 @@ gid_t group2gid(char *name)
 /*
  *	Convert name to id
  */
-int name2id(char *name, int qtype)
+int name2id(char *name, int qtype, int *err)
 {
 	if (qtype == USRQUOTA)
-		return user2uid(name);
+		return user2uid(name, err);
 	else
-		return group2gid(name);
+		return group2gid(name, err);
 }
 
 /*
@@ -376,12 +414,25 @@ static int hasxfsquota(struct mntent *mnt, int type)
 	return ret;
 }
 
+/* Return if given option has nonempty argument */
+char *hasmntoptarg(struct mntent *mnt, char *opt)
+{
+	char *p = hasmntopt(mnt, opt);
+
+	if (!p)
+		return NULL;
+	p += strlen(opt);
+	if (*p == '=' && p[1] != ',')
+		return p+1;
+	return NULL;
+}
+
 /*
  *	Check to see if a particular quota is to be enabled (filesystem mounted with proper option)
  */
 int hasquota(struct mntent *mnt, int type)
 {
-	if (!CORRECT_FSTYPE(mnt->mnt_type) || hasmntopt(mnt, MNTOPT_NOQUOTA))
+	if (!correct_fstype(mnt->mnt_type) || hasmntopt(mnt, MNTOPT_NOQUOTA))
 		return 0;
 	
 	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS))
@@ -389,9 +440,9 @@ int hasquota(struct mntent *mnt, int type)
 	if (!strcmp(mnt->mnt_type, MNTTYPE_NFS))	/* NFS always has quota or better there is no good way how to detect it */
 		return 1;
 
-	if ((type == USRQUOTA) && hasmntopt(mnt, MNTOPT_USRQUOTA))
+	if ((type == USRQUOTA) && (hasmntopt(mnt, MNTOPT_USRQUOTA) || hasmntoptarg(mnt, MNTOPT_USRJQUOTA)))
 		return 1;
-	if ((type == GRPQUOTA) && hasmntopt(mnt, MNTOPT_GRPQUOTA))
+	if ((type == GRPQUOTA) && (hasmntopt(mnt, MNTOPT_GRPQUOTA) || hasmntoptarg(mnt, MNTOPT_GRPJQUOTA)))
 		return 1;
 	if ((type == USRQUOTA) && hasmntopt(mnt, MNTOPT_QUOTA))
 		return 1;
@@ -442,10 +493,22 @@ int get_qf_name(struct mntent *mnt, int type, int fmt, int flags, char **filenam
 		if (*(pathname = option + strlen(MNTOPT_USRQUOTA)) == '=')
 			has_quota_file_definition = 1;
 	}
+	else if (type == USRQUOTA && (option = hasmntoptarg(mnt, MNTOPT_USRJQUOTA))) {
+		pathname = option-1;
+		has_quota_file_definition = 1;
+		sstrncpy(qfullname, mnt->mnt_dir, sizeof(qfullname));
+		sstrncat(qfullname, "/", sizeof(qfullname));
+	}
 	else if (type == GRPQUOTA && (option = hasmntopt(mnt, MNTOPT_GRPQUOTA))) {
 		if (*(pathname = option + strlen(MNTOPT_GRPQUOTA)) == '=')
 			has_quota_file_definition = 1;
 	}
+	else if (type == GRPQUOTA && (option = hasmntoptarg(mnt, MNTOPT_GRPJQUOTA))) {
+		pathname = option-1;
+		has_quota_file_definition = 1;
+		sstrncpy(qfullname, mnt->mnt_dir, sizeof(qfullname));
+		sstrncat(qfullname, "/", sizeof(qfullname));
+	}
 	else if (type == USRQUOTA && (option = hasmntopt(mnt, MNTOPT_QUOTA))) {
 		if (*(pathname = option + strlen(MNTOPT_QUOTA)) == '=')
 			has_quota_file_definition = 1;
@@ -455,9 +518,9 @@ int get_qf_name(struct mntent *mnt, int type, int fmt, int flags, char **filenam
 
 	if (has_quota_file_definition) {
 		if ((option = strchr(++pathname, ',')))
-			sstrncpy(qfullname, pathname, min((option - pathname + 1), sizeof(qfullname)));
+			sstrncpy(qfullname+strlen(qfullname), pathname, min((option - pathname + 1), sizeof(qfullname)-strlen(qfullname)));
 		else
-			sstrncpy(qfullname, pathname, sizeof(qfullname));
+			sstrncat(qfullname, pathname, sizeof(qfullname));
 	}
 	if (fmt & (1 << QF_VFSV0)) {
 		if (!has_quota_file_definition)
@@ -564,29 +627,41 @@ int devcmp_handles(struct quota_handle *a, struct quota_handle *b)
 
 int kernel_iface, kernel_formats;	/* Formats supported by kernel */
 
+#ifndef FS_DQSTATS
+#define FS_DQSTATS 16
+#endif
+#ifndef FS_DQ_SYNCS
+#define FS_DQ_SYNCS 8
+#endif
+
 void init_kernel_interface(void)
 {
 	struct stat st;
+	struct sigaction sig, oldsig;
 	
+	/* This signal handling is needed because old kernels send us SIGSEGV as they try to resolve the device */
+	sig.sa_handler = SIG_IGN;
+	sig.sa_sigaction = NULL;
+	if (sigemptyset(&sig.sa_mask) < 0)
+		die(2, _("Can't create set for sigaction(): %s\n"), strerror(errno));
+	sig.sa_flags = 0;
+	if (sigaction(SIGSEGV, &sig, &oldsig) < 0)
+		die(2, _("Can't set signal handler: %s\n"), strerror(errno));
+
 	kernel_formats = 0;
 	if (!stat("/proc/fs/xfs/stat", &st))
 		kernel_formats |= (1 << QF_XFS);
-	if (!stat("/proc/sys/fs/quota", &st)) {
+	else
+		if (!quotactl(QCMD(Q_XGETQSTAT, 0), NULL, 0, NULL) || (errno != EINVAL && errno != ENOSYS))
+			kernel_formats |= (1 << QF_XFS);
+	/* Detect new kernel interface; Assume generic interface unless we can prove there is not one... */
+	if (!stat("/proc/sys/fs/quota", &st) || errno != ENOENT) {
 		kernel_iface = IFACE_GENERIC;
 		kernel_formats |= (1 << QF_VFSOLD) | (1 << QF_VFSV0);
 	}
 	else {
 		struct v2_dqstats v2_stats;
-		struct sigaction sig, oldsig;
 
-		/* This signal handling is needed because old kernels send us SIGSEGV as they try to resolve the device */
-		sig.sa_handler = SIG_IGN;
-		sig.sa_sigaction = NULL;
-		if (sigemptyset(&sig.sa_mask) < 0)
-			die(2, _("Can't create set for sigaction(): %s\n"), strerror(errno));
-		sig.sa_flags = 0;
-		if (sigaction(SIGSEGV, &sig, &oldsig) < 0)
-			die(2, _("Can't set signal handler: %s\n"), strerror(errno));
 		if (quotactl(QCMD(Q_V2_GETSTATS, 0), NULL, 0, (void *)&v2_stats) >= 0) {
 			kernel_formats |= (1 << QF_VFSV0);
 			kernel_iface = IFACE_VFSV0;
@@ -616,9 +691,9 @@ void init_kernel_interface(void)
 				kernel_iface = IFACE_VFSOLD;
 			}
 		}
-		if (sigaction(SIGSEGV, &oldsig, NULL) < 0)
-			die(2, _("Can't reset signal handler: %s\n"), strerror(errno));
 	}
+	if (sigaction(SIGSEGV, &oldsig, NULL) < 0)
+		die(2, _("Can't reset signal handler: %s\n"), strerror(errno));
 }
 
 /* Check whether old quota is turned on on given device */
@@ -769,7 +844,7 @@ static int cache_mnt_table(int flags)
 		
 		/* Further we are not interested in mountpoints without quotas and
 		   we don't want to touch them */
-		if (!CORRECT_FSTYPE(mnt->mnt_type) || hasmntopt(mnt, MNTOPT_NOQUOTA) || !(hasmntopt(mnt, MNTOPT_USRQUOTA) || hasmntopt(mnt, MNTOPT_GRPQUOTA) || hasmntopt(mnt, MNTOPT_QUOTA) || !strcmp(mnt->mnt_type, MNTTYPE_NFS))) {
+		if (!hasquota(mnt, USRQUOTA) && !hasquota(mnt, GRPQUOTA)) {
 			free((char *)devname);
 			continue;
 		}
@@ -797,7 +872,12 @@ static int cache_mnt_table(int flags)
 				free((char *)devname);
 				continue;
 			}
-			if (!S_ISBLK(st.st_mode) && !S_ISCHR(st.st_mode)) {	/* Some 'bind' or 'loop' mount? */
+			if (!S_ISBLK(st.st_mode) && !S_ISCHR(st.st_mode) && !S_ISREG(st.st_mode) && !S_ISDIR(st.st_mode)) {
+				unsupporteddev:
+				errstr(_("Device (%s) filesystem is mounted on unsupported device type. Skipping.\n"), devname);
+				free((char *)devname);
+				continue;
+			} else {
 				char *opt;
 				
 				if (hasmntopt(mnt, MNTOPT_BIND)) {
@@ -830,15 +910,12 @@ static int cache_mnt_table(int flags)
 					dev = st.st_rdev;
 					free((char *)devname);
 					devname = sstrdup(loopdev);
-				}
-				else {
-					errstr(_("Device (%s) filesystem is mounted on isn't block or character device nor it's loopback or bind mount. Skipping.\n"), devname);
-					free((char *)devname);
-					continue;
+				} else {
+					if (!S_ISBLK(st.st_mode) && !S_ISCHR(st.st_mode))
+						goto unsupporteddev;
+					dev = st.st_rdev;
 				}
 			}
-			else
-				dev = st.st_rdev;
 			for (i = 0; i < mnt_entries_cnt && mnt_entries[i].me_dev != dev; i++);
 		}
 		/* Cope with network filesystems or new mountpoint */
@@ -910,7 +987,8 @@ static int process_dirs(int dcnt, char **dirs, int flags)
 
 				/* Return st of mountpoint of dir in st.. */
 				if (flags & MS_NO_MNTPOINT && !(realmnt = find_dir_mntpoint(&st))) {
-					errstr(_("Can't find filesystem mountpoint for directory %s\n"), dirs[i]);
+					if (!(flags & MS_QUIET))
+						errstr(_("Can't find filesystem mountpoint for directory %s\n"), dirs[i]);
 					continue;
 				}
 				check_dirs[check_dirs_cnt].sd_dev = st.st_dev;
@@ -926,7 +1004,8 @@ static int process_dirs(int dcnt, char **dirs, int flags)
 				check_dirs[check_dirs_cnt].sd_dev = st.st_rdev;
 				for (mentry = 0; mentry < mnt_entries_cnt && mnt_entries[mentry].me_dev != st.st_rdev; mentry++);
 				if (mentry == mnt_entries_cnt) {
-					errstr(_("Can't find mountpoint for device %s\n"), dirs[i]);
+					if (!(flags & MS_QUIET))
+						errstr(_("Can't find mountpoint for device %s\n"), dirs[i]);
 					continue;
 				}
 				sstrncpy(mntpointbuf, mnt_entries[mentry].me_dir, PATH_MAX-1);
@@ -939,7 +1018,8 @@ static int process_dirs(int dcnt, char **dirs, int flags)
 			check_dirs_cnt++;
 		}
 		if (!check_dirs_cnt) {
-			errstr(_("No correct mountpoint specified.\n"));
+			if (!(flags & MS_QUIET))
+				errstr(_("No correct mountpoint specified.\n"));
 			free(check_dirs);
 			return -1;
 		}
