@@ -41,13 +41,13 @@ extern int kernel_formats, kernel_iface;
 char *type2name(int);
 
 /* Convert username to uid */
-uid_t user2uid(char *);
+uid_t user2uid(char *, int *err);
 
 /* Convert groupname to gid */
-gid_t group2gid(char *);
+gid_t group2gid(char *, int *err);
 
 /* Convert user/groupname to id */
-int name2id(char *name, int qtype);
+int name2id(char *name, int qtype, int *err);
 
 /* Convert uid to username */
 int uid2user(uid_t, char *);
@@ -124,6 +124,7 @@ int kern_quota_on(const char *dev, int type, int fmt);
 /* Flags for init_mounts_scan() */
 #define MS_NO_MNTPOINT 0x01	/* Specified directory needn't be mountpoint */
 #define MS_NO_AUTOFS 0x02	/* Ignore autofs mountpoints */
+#define MS_QUIET 0x04		/* Be quiet with error reporting */
 /* Initialize mountpoints scan */
 int init_mounts_scan(int dcnt, char **dirs, int flags);
 
